clear
addpath('func');
REPS=10000;
BURN=8000;
T0=40;   %training sample



[data0,names]=xlsread('data103.xls'); %load panel of 30 variables for the UK Bank rate is the 17th variable see index.xls
N=cols(data0);
data0=detrend(data0);
data=data0;
L=2; %lag length of the VAR
Y=data;
trend=(0:length(Y)-1)';

%take lags
X=[];
for j=1:L
X=[X lag0(data,j) ];
end
X=[X ones(rows(X),1) trend];
Y=Y(L+1:end,:);
X=X(L+1:end,:);
T=rows(X);

%initial training sample
Y0=Y(1:T0,:);
X0=X(1:T0,:);


%priors for VAR coefficients
lamdaP=0.1;  %This controls the tightness of the priors on the first lag
tauP=10*lamdaP;  % this controls the tightness of the priors on sum of coefficients
epsilonP=1/1000;  % this controls tightness of the prior on the constant
muP=mean(Y0)';
sigmaP=[];
deltaP=[];
e0=[];
for i=1:N
    ytemp=Y0(:,i);
    xtemp=[lag0(ytemp,1) ones(rows(ytemp),1)];
    ytemp=ytemp(2:end,:);
    xtemp=xtemp(2:end,:);
    btemp=xtemp\ytemp;
    etemp=ytemp-xtemp*btemp;
    stemp=etemp'*etemp/rows(ytemp);
    if btemp(1)>1
        btemp(1)=1;
    end
    deltaP=[deltaP;btemp(1)];
    sigmaP=[sigmaP;sqrt(stemp)];
    e0=[e0 etemp];
end
%dummy data to implement priors see http://ideas.repec.org/p/ecb/ecbwps/20080966.html
[yd,xd] = create_dummiestrend(lamdaP,tauP,deltaP,epsilonP,L,muP,sigmaP,N);

%prior mean and variance using the dummy data
B0=xd\yd;
E0=yd-xd*B0;
 SIGMA0=kron(E0'*E0,pinv(xd'*xd)); %prior variance
% SIGMA0=kron(eye(N),pinv(xd'*xd)); %prior variance

B0=vec(B0); %prior mean


%priors for the elements of A matrix
s0=(e0'*e0)/T0;
C0=chol(s0);
C0=inv(C0./repmat(diag(C0),1,N))'; %prior mean
SC0=10;  %prior variance

%priors for the initial condition of the stochastic volatility
MU0=log(diag(s0)); %prior mean
SV0=1;           %prior variance

%Remove training sample. This is the data used for estimation
  Y=Y(T0+1:end,:);
  X=X(T0+1:end,:);
  T=rows(Y);
  %rough guess for stochastic volatility
 
hlast=(diff(Y).^2)+0.0001;
hlast=[hlast(1:2,:);hlast];  %rough intial guess for svol
errors=diff(Y); %initial guess for VAR residuals
errors=[errors(1,:);errors(1:end,:)];
g=ones(N,1);  %rough guess for the variance of the transition equation
g0=0.01^2;  %prior scale parameter for inverse gamma
Tg0=1;     %prior degrees of freedom
SV=zeros(REPS-BURN,T+1,N);


B0h=[0.7;0];
Sigma0h=diag([0.5 0.5]);

MUSVOL=zeros(N,1);
FSVOL=zeros(N,1);








tic;
jgibbs=1;
for igibbs=1:REPS
    igibbs
%step 1 of the Gibbs algorithm. Sample the A matrix  (See step 3 in the pdf
%file)

 amatx=[];

for j=2:N
    
%v2=-a1*v1+sqrt(exp(h2))*e2
ytemp=errors(:,j);  
xtemp=errors(:,1:j-1)*(-1); 
ytemp=ytemp./sqrt(hlast(2:rows(hlast),j)); %remove heteroscedasticity
xtemp=xtemp./repmat(sqrt(hlast(2:rows(hlast),j)),1,cols(xtemp));%remove heteroscedasticity
%prior means and variances
A0=C0(j,1:j-1)';
V00=diag(abs(A0))*SC0;

atemp=getreg(ytemp,xtemp,A0,V00,1); 

amatx=[amatx atemp'];
end
  
%step 2 of the algorithm: Sample stochastic volatilities  
A=chofac(N,amatx');
epsilon=errors*A';  %orthogonal residuals
%sample stochastic vol for each epsilon using the MH algorithm in Jaquier
%Polson and Rossi
  hnew=[];
  for i=1:N
      htemp=getsvol(hlast(:,i),g(i),MU0(i),SV0,epsilon(:,i),MUSVOL(i),FSVOL(i)); 
      
      hnew=[hnew htemp];
  end
hlast=hnew;




%draw AR parameters
 gerrors=[];
 for jj=1:N
     ytemp=log(hlast(:,jj));
     xtemp=[lag0(ytemp,1) ones(rows(ytemp),1)];
     ytemp=ytemp(2:end,:);
     xtemp=xtemp(2:end,:);


MM=inv(inv(Sigma0h)+(1/g(jj))*(xtemp'*xtemp))*(inv(Sigma0h)*B0h+(1/g(jj))*xtemp'*ytemp); 
VV=inv(inv(Sigma0h)+(1/g(jj))*(xtemp'*xtemp));
chck=-1;
while chck<0                     %check for stability
BB=MM+(randn(1,2)*chol(VV))';
ee=max(abs(BB(1)));
if ee<=1
    chck=1;
end
end
     FSVOL(jj)=BB(1);
     MUSVOL(jj)=BB(2);
     gerrors=[gerrors ytemp-xtemp*BB];
 end













 
 %step 3 of the algorithm: Sample g from the IG distribution
 for i=1:N
    
g(i)=ig(Tg0,g0,gerrors(:,i));  %draw from the inverse Gamma distribution
 end

 %step 4: Sample VAR coefficients
%  [beta2,errors]=kcohn(Y,X,amatx,hlast,B0',SIGMA0,L);  %Conditional distribution of the VAR coefficients using the Kalman filter
beta2=[X;xd]\[Y;yd];
errors=Y-X*beta2;
beta2=vec(beta2);
 
 if igibbs>BURN
     out1(jgibbs,:)=beta2;
    out2(jgibbs,1:T,1:N)=hlast(2:end,:);
    out4(jgibbs,1:T,1:(N*(N-1))/2)=repmat(amatx,T,1);
 
    out7(jgibbs,1:N)=g';
    out8(jgibbs,1:N*2)=[FSVOL' MUSVOL'];
     jgibbs=jgibbs+1;
 end
end

   toc; 
   
   
   
   
   
   
   
   
   save tvp.mat out1 out2 out4   out7 out8



%produce starting values for main algorithm
hmean=log(squeeze(median(out2,1)));
amean=squeeze(mean(out4,1));
YY=[Y(1:end,:) ];
trend=(0:length(YY)-1)';
XX=[ lag0(YY,1) lag0(YY,2)  ((squeeze(median(log(out2),1)))) lag0(((squeeze(median(log(out2),1)))),1)    ones(size(YY,1),1) ];
YY=YY(5:end,:);
XX=XX(5:end,:);












betaols=(inv(XX'*XX)*(XX'*YY));
eols=YY-XX*betaols;
Sigma=(eols'*eols)/(rows(eols));
% pattern=xlsread('pattern.xlsx');
% pattern=vec(pattern);
% 
% id=find(pattern==0);
% NN=rows(vec(betaols));
% b0=zeros(NN,1);
% H=eye(NN)*1000;
% for k=1:length(id);
%     H(id(k),id(k))=1e-19;
% end
% 
% M=inv(inv(H)+kron(inv(Sigma),XX'*XX))*(inv(H)*b0+kron(inv(Sigma),XX'*XX)*vec(betaols));










gout=mean(out7);


%produce initial state vector
temp=(chofac(N,amean(end,:)'));
temp1=amean(end,:);

% temp1(:,[1 2 4])=abs(temp1(:,[1 2 4]));


beta0=[hmean(2,:)';hmean(1,:)';  temp1'];




save initial1.mat betaols
save initial2.mat gout

save initial4.mat beta0
temp=squeeze(median(out2,1));
temp=[temp(1,:); temp];
save initial5.mat  temp
temp=mean(out8);
save initial6.mat  temp



dlmwrite('initial1.txt',betaols,'delimiter','\t','precision','%.6f')

dlmwrite('initial2.txt',gout,'delimiter','\t','precision','%.6f')

dlmwrite('initial4.txt',beta0,'delimiter','\t','precision','%.6f')
temp=squeeze(median(out2,1));
temp=[temp(1,:); temp];

dlmwrite('initial5.txt',temp,'delimiter','\t','precision','%.6f')
temp=mean(out8);
dlmwrite('initial6.txt',temp,'delimiter','\t','precision','%.6f')

dlmwrite('datain.txt',data0,'delimiter','\t','precision','%.20f')

   
   
   
   
   
   
   
   
   
   
   